@@ -1,19 +1,22 @@
-import os
 import glob
 import json
-from typing import List
+import os
+import re
 from logging import getLogger
-from PIL import Image
+from typing import List
+
 import cv2
 import numpy as np
-
 import xmltodict
+from PIL import Image
 
-from .exceptions import FastLabelInvalidException
-from .api import Api
-from fastlabel import converters, utils, const
+from fastlabel import const, converters, utils
 from fastlabel.const import AnnotationType
 
+from .api import Api
+from .exceptions import FastLabelInvalidException
+
+
 logger = getLogger(__name__)
 
 
@@ -349,6 +352,7 @@ class Client:
         external_status: str = None,
         annotations: list = [],
         tags: list = [],
+        **kwargs
     ) -> str:
         """
         Create a single image task.
@@ -360,6 +364,12 @@ class Client:
         external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         annotations is a list of annotation to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
+        assignee is slug of assigned user. (Optional)
+        reviewer is slug of review user. (Optional)
+        approver is slug of approve user. (Optional)
+        external_assignee is slug of external assigned user. (Optional)
+        external_reviewer is slug of external review user. (Optional)
+        external_approver is slug of external approve user. (Optional)
         """
         endpoint = "tasks/image"
         if not utils.is_image_supported_ext(file_path):
@@ -377,6 +387,9 @@ class Client:
             payload["annotations"] = annotations
         if tags:
             payload["tags"] = tags
+
+        self.__fill_assign_users(payload, **kwargs)
+
         return self.api.post_request(endpoint, payload=payload)
 
     def create_image_classification_task(
@@ -388,6 +401,7 @@ class Client:
         external_status: str = None,
         attributes: list = [],
         tags: list = [],
+        **kwargs,
     ) -> str:
         """
         Create a single image classification task.
@@ -399,6 +413,12 @@ class Client:
         external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         attributes is a list of attribute to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
+        assignee is slug of assigned user. (Optional)
+        reviewer is slug of review user. (Optional)
+        approver is slug of approve user. (Optional)
+        external_assignee is slug of external assigned user. (Optional)
+        external_reviewer is slug of external review user. (Optional)
+        external_approver is slug of external approve user. (Optional)
         """
         endpoint = "tasks/image/classification"
         if not utils.is_image_supported_ext(file_path):
@@ -414,6 +434,9 @@ class Client:
             payload["attributes"] = attributes
         if tags:
             payload["tags"] = tags
+
+        self.__fill_assign_users(payload, **kwargs)
+
         return self.api.post_request(endpoint, payload=payload)
 
     def create_multi_image_task(
@@ -425,6 +448,7 @@ class Client:
         external_status: str = None,
         annotations: list = [],
         tags: list = [],
+        **kwargs,
     ) -> str:
         """
         Create a single multi image task.
@@ -436,6 +460,12 @@ class Client:
         external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         annotations is a list of annotation to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
+        assignee is slug of assigned user. (Optional)
+        reviewer is slug of review user. (Optional)
+        approver is slug of approve user. (Optional)
+        external_assignee is slug of external assigned user. (Optional)
+        external_reviewer is slug of external review user. (Optional)
+        external_approver is slug of external approve user. (Optional)
         """
         if not os.path.isdir(folder_path):
             raise FastLabelInvalidException(
@@ -465,6 +495,9 @@ class Client:
             payload["annotations"] = annotations
         if tags:
             payload["tags"] = tags
+
+        self.__fill_assign_users(payload, **kwargs)
+
         return self.api.post_request(endpoint, payload=payload)
 
     def create_video_task(
@@ -476,6 +509,7 @@ class Client:
         external_status: str = None,
         annotations: list = [],
         tags: list = [],
+        **kwargs,
     ) -> str:
         """
         Create a single video task.
@@ -487,6 +521,12 @@ class Client:
         external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         annotations is a list of annotation to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
+        assignee is slug of assigned user. (Optional)
+        reviewer is slug of review user. (Optional)
+        approver is slug of approve user. (Optional)
+        external_assignee is slug of external assigned user. (Optional)
+        external_reviewer is slug of external review user. (Optional)
+        external_approver is slug of external approve user. (Optional)
         """
         endpoint = "tasks/video"
         if not utils.is_video_supported_ext(file_path):
@@ -504,6 +544,9 @@ class Client:
             payload["annotations"] = annotations
         if tags:
             payload["tags"] = tags
+
+        self.__fill_assign_users(payload, **kwargs)
+
         return self.api.post_request(endpoint, payload=payload)
 
     def create_video_classification_task(
@@ -515,6 +558,7 @@ class Client:
         external_status: str = None,
         attributes: list = [],
         tags: list = [],
+        **kwargs,
     ) -> str:
         """
         Create a single video classification task.
@@ -526,10 +570,17 @@ class Client:
         external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         attributes is a list of attribute to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
+        assignee is slug of assigned user. (Optional)
+        reviewer is slug of review user. (Optional)
+        approver is slug of approve user. (Optional)
+        external_assignee is slug of external assigned user. (Optional)
+        external_reviewer is slug of external review user. (Optional)
+        external_approver is slug of external approve user. (Optional)
         """
         endpoint = "tasks/video/classification"
         if not utils.is_video_supported_ext(file_path):
-            raise FastLabelInvalidException("Supported extensions are mp4.", 422)
+            raise FastLabelInvalidException(
+                "Supported extensions are mp4.", 422)
         file = utils.base64_encode(file_path)
         payload = {"project": project, "name": name, "file": file}
         if status:
@@ -540,6 +591,9 @@ class Client:
             payload["attributes"] = attributes
         if tags:
             payload["tags"] = tags
+
+        self.__fill_assign_users(payload, **kwargs)
+
         return self.api.post_request(endpoint, payload=payload)
 
     # Task Update
@@ -550,6 +604,7 @@ class Client:
         status: str = None,
         external_status: str = None,
         tags: list = [],
+        **kwargs,
     ) -> str:
         """
         Update a single task.
@@ -558,6 +613,12 @@ class Client:
         status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined'. (Optional)
         external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         tags is a list of tag to be set. (Optional)
+        assignee is slug of assigned user. (Optional)
+        reviewer is slug of review user. (Optional)
+        approver is slug of approve user. (Optional)
+        external_assignee is slug of external assigned user. (Optional)
+        external_reviewer is slug of external review user. (Optional)
+        external_approver is slug of external approve user. (Optional)
         """
         endpoint = "tasks/" + task_id
         payload = {}
@@ -567,6 +628,9 @@ class Client:
             payload["externalStatus"] = external_status
         if tags:
             payload["tags"] = tags
+
+        self.__fill_assign_users(payload, **kwargs)
+
         return self.api.put_request(endpoint, payload=payload)
 
     def update_image_classification_task(
@@ -576,6 +640,7 @@ class Client:
         external_status: str = None,
         attributes: list = [],
         tags: list = [],
+        **kwargs,
     ) -> str:
         """
         Create a single image classification task.
@@ -585,6 +650,12 @@ class Client:
         external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         attributes is a list of attribute to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
+        assignee is slug of assigned user. (Optional)
+        reviewer is slug of review user. (Optional)
+        approver is slug of approve user. (Optional)
+        external_assignee is slug of external assigned user. (Optional)
+        external_reviewer is slug of external review user. (Optional)
+        external_approver is slug of external approve user. (Optional)
         """
         endpoint = "tasks/image/classification/" + task_id
         payload = {}
@@ -596,6 +667,9 @@ class Client:
             payload["attributes"] = attributes
         if tags:
             payload["tags"] = tags
+
+        self.__fill_assign_users(payload, **kwargs)
+
         return self.api.put_request(endpoint, payload=payload)
 
     def update_video_classification_task(
@@ -605,6 +679,7 @@ class Client:
         external_status: str = None,
         attributes: list = [],
         tags: list = [],
+        **kwargs,
     ) -> str:
         """
         Create a single video classification task.
@@ -614,6 +689,12 @@ class Client:
         external_status can be 'registered', 'completed', 'skipped', 'reviewed', 'sent_back', 'approved', 'declined',  'customer_declined'. (Optional)
         attributes is a list of attribute to be set in advance. (Optional)
         tags is a list of tag to be set in advance. (Optional)
+        assignee is slug of assigned user. (Optional)
+        reviewer is slug of review user. (Optional)
+        approver is slug of approve user. (Optional)
+        external_assignee is slug of external assigned user. (Optional)
+        external_reviewer is slug of external review user. (Optional)
+        external_approver is slug of external approve user. (Optional)
         """
         endpoint = "tasks/video/classification/" + task_id
         payload = {}
@@ -625,6 +706,9 @@ class Client:
             payload["attributes"] = attributes
         if tags:
             payload["tags"] = tags
+
+        self.__fill_assign_users(payload, **kwargs)
+
         return self.api.put_request(endpoint, payload=payload)
 
     # Task Delete
@@ -636,6 +720,305 @@ class Client:
         endpoint = "tasks/" + task_id
         self.api.delete_request(endpoint)
 
+    # Convert to Fastlabel
+
+    def convert_coco_to_fastlabel(self, file_path: str) -> dict:
+        """
+        Convert COCO format to FastLabel format as annotation file.
+
+        file_path is a COCO format annotation file. (Required)
+
+        In the output file, the key is the image file name and the value is a list of annotations in FastLabel format, which is returned in dict format.
+
+        output format example.
+        {
+            'sample1.jpg':  [
+                {
+                    'points': [
+                        100,
+                        100,
+                        200,
+                        200
+                    ],
+                    'type': 'bbox',
+                    'value': 'cat'
+                }
+            ],
+            'sample2.jpg':  [
+                {
+                    'points': [
+                        100,
+                        100,
+                        200,
+                        200
+                    ],
+                    'type': 'bbox',
+                    'value': 'cat'
+                }
+            ]
+        }
+        """
+        with open(file_path, "r") as f:
+            file = f.read()
+            return converters.execute_coco_to_fastlabel(eval(file))
+
+    def convert_labelme_to_fastlabel(self, folder_path: str) -> dict:
+        """
+        Convert labelme format to FastLabel format as annotation files.
+
+        folder_path is the folder that contains the labelme format files with the json extension. (Required)
+
+        In the output file, the key is the image file name and the value is a list of annotations in FastLabel format, which is returned in dict format.
+        If the tree has multiple hierarchies, the key is the relative path rooted at the specified folder name.
+
+        output format example.
+        In the case of labelme, the key is the tree structure if the tree structure is multi-level.
+
+        [tree structure]
+        dataset
+        ├── sample1.jpg
+        ├── sample1.json
+        └── sample_dir
+            ├── sample2.jpg
+            └── sample2.json
+
+        [output]
+        {
+            'sample1.jpg':  [
+                {
+                    'points': [
+                        100,
+                        100,
+                        200,
+                        200
+                    ],
+                    'type': 'bbox',
+                    'value': 'cat'
+                }
+            ],
+            'sample_dir/sample2.jpg':  [
+                {
+                    'points': [
+                        100,
+                        100,
+                        200,
+                        200
+                    ],
+                    'type': 'bbox',
+                    'value': 'cat'
+                    }
+            ]
+        }
+        """
+        results = {}
+        for file_path in glob.iglob(
+            os.path.join(folder_path, "**/**.json"), recursive=True
+        ):
+            with open(file_path, "r") as f:
+                c = converters.execute_labelme_to_fastlabel(
+                    json.load(f),
+                    file_path.replace(os.path.join(*[folder_path, ""]), ""),
+                )
+                results[c[0]] = c[1]
+        return results
+
+    def convert_pascalvoc_to_fastlabel(self, folder_path: str) -> dict:
+        """
+        Convert PascalVOC format to FastLabel format as annotation files.
+
+        folder_path is the folder that contains the PascalVOC format files with the xml extension. (Required)
+
+        In the output file, the key is the image file name and the value is a list of annotations in FastLabel format, which is returned in dict format.
+        If the tree has multiple hierarchies, the key is the relative path rooted at the specified folder name.
+
+        output format example.
+        In the case of PascalVOC, the key is the tree structure if the tree structure is multi-level.
+
+        [tree structure]
+        dataset
+        ├── sample1.jpg
+        ├── sample1.xml
+        └── sample_dir
+            ├── sample2.jpg
+            └── sample2.xml
+
+        [output]
+        {
+            'sample1.jpg':  [
+                {
+                    'points': [
+                        100,
+                        100,
+                        200,
+                        200
+                    ],
+                    'type': 'bbox',
+                    'value': 'cat'
+                }
+            ],
+            'sample_dir/sample2.jpg':  [
+                {
+                    'points': [
+                        100,
+                        100,
+                        200,
+                        200
+                    ],
+                    'type': 'bbox',
+                    'value': 'cat'
+                }
+            ]
+        }
+        """
+        results = {}
+        for file_path in glob.iglob(
+            os.path.join(folder_path, "**/**.xml"), recursive=True
+        ):
+            with open(file_path, "r") as f:
+                file = f.read()
+                c = converters.execute_pascalvoc_to_fastlabel(
+                    xmltodict.parse(file),
+                    file_path.replace(os.path.join(*[folder_path, ""]), ""),
+                )
+                results[c[0]] = c[1]
+        return results
+
+    def convert_yolo_to_fastlabel(
+        self, classes_file_path: str, dataset_folder_path: str
+    ) -> dict:
+        """
+        Convert YOLO format to FastLabel format as annotation files.
+
+        classes_file_path is YOLO format class file. (Required)
+        dataset_folder_path is the folder that contains the image file and YOLO format files with the txt extension. (Required)
+
+        In the output file, the key is the image file name and the value is a list of annotations in FastLabel format, which is returned in dict format.
+        If the tree has multiple hierarchies, the key is the relative path rooted at the specified folder name.
+
+        output format example.
+        In the case of YOLO, the key is the tree structure if the tree structure is multi-level.
+
+        [tree structure]
+        dataset
+        ├── sample1.jpg
+        ├── sample1.txt
+        └── sample_dir
+            ├── sample2.jpg
+            └── sample2.txt
+
+        [output]
+        {
+            'sample1.jpg':  [
+                {
+                    'points': [
+                        100,
+                        100,
+                        200,
+                        200
+                    ],
+                    'type': 'bbox',
+                    'value': 'cat'
+                }
+            ],
+            'sample_dir/sample2.jpg':  [
+                {
+                    'points': [
+                        100,
+                        100,
+                        200,
+                        200
+                    ],
+                    'type': 'bbox',
+                    'value': 'cat'
+                }
+            ]
+        }
+        """
+        classes = self.__get_yolo_format_classes(classes_file_path)
+        image_sizes = self.__get_yolo_image_sizes(dataset_folder_path)
+        yolo_annotations = self.__get_yolo_format_annotations(
+            dataset_folder_path)
+
+        return converters.execute_yolo_to_fastlabel(
+            classes,
+            image_sizes,
+            yolo_annotations,
+            os.path.join(*[dataset_folder_path, ""]),
+        )
+
+    def __get_yolo_format_classes(self, classes_file_path: str) -> dict:
+        """
+        return data format
+        {
+            id: classs_name
+            ...
+        }
+        """
+        classes = {}
+        with open(classes_file_path, "r") as f:
+            lines = f.readlines()
+            line_index = 0
+            for line in lines:
+                classes[str(line_index)] = line.strip()
+                line_index += 1
+        return classes
+
+    def __get_yolo_image_sizes(self, dataset_folder_path: str) -> dict:
+        """
+        return data format
+        {
+            image_file_path_without_ext: {
+                "image_file_path": image file full path
+                "size": [whdth, height]
+            ...
+        }
+        """
+        image_types = utils.get_supported_image_ext()
+        image_paths = [
+            p for p in glob.glob(os.path.join(dataset_folder_path, "**/*"), recursive=True)
+            if re.search("/*\.({})".format("|".join(image_types)), str(p))
+        ]
+        image_sizes = {}
+        for image_path in image_paths:
+            image = Image.open(image_path)
+            width, height = image.size
+            image_sizes[image_path.replace(os.path.splitext(image_path)[1], "")] = {
+                "image_file_path": image_path,
+                "size": [width, height],
+            }
+
+        return image_sizes
+
+    def __get_yolo_format_annotations(self, dataset_folder_path: str) -> dict:
+        """
+        return data format
+        {
+            annotaion_file_path_without_ext:
+                [
+                    yolo_class_id,
+                    yolo_center_x_ratio,
+                    yolo_center_y_ratio,
+                    yolo_anno_width_ratio,
+                    yolo_anno_height_ratio
+                ],
+            ...
+        }
+        """
+        yolo_annotations = {}
+        annotaion_file_paths = [
+            p for p in glob.glob(os.path.join(dataset_folder_path, "**/*.txt"), recursive=True)
+            if re.search(("/*\.txt"), str(p))
+        ]
+        for annotaion_file_path in annotaion_file_paths:
+            with open(annotaion_file_path, "r") as f:
+                anno_lines = f.readlines()
+                annotaion_key = annotaion_file_path.replace(".txt", "")
+                yolo_annotations[annotaion_key] = []
+                for anno_line in anno_lines:
+                    yolo_annotations[annotaion_key].append(
+                        anno_line.strip().split(" "))
+        return yolo_annotations
+
     # Task Convert
 
     def export_coco(self, tasks: list, output_dir: str = os.path.join("output", "coco")) -> None:
@@ -651,14 +1034,17 @@ class Client:
         with open(file_path, 'w') as f:
             json.dump(coco, f, indent=4, ensure_ascii=False)
 
-    def export_yolo(self, tasks: list, output_dir: str = os.path.join("output", "yolo")) -> None:
+    def export_yolo(self, tasks: list, classes: list = [], output_dir: str = os.path.join("output", "yolo")) -> None:
         """
         Convert tasks to YOLO format and export as files.
+        If you pass classes, classes.txt will be generated based on it .
+        If not , classes.txt will be generated based on passed tasks .(Annotations never used in your project will not be exported.)
 
         tasks is a list of tasks. (Required)
+        classes is a list of annotation values.  e.g. ['dog','bird'] (Optional)
         output_dir is output directory(default: output/yolo). (Optional)
         """
-        annos, categories = converters.to_yolo(tasks)
+        annos, categories = converters.to_yolo(tasks, classes)
         for anno in annos:
             file_name = anno["filename"]
             basename = utils.get_basename(file_name)
@@ -709,12 +1095,12 @@ class Client:
             with open(file_path, 'w') as f:
                 json.dump(labelme, f, indent=4, ensure_ascii=False)
 
-
     # Instance / Semantic Segmetation
+
     def export_instance_segmentation(self, tasks: list, output_dir: str = os.path.join("output", "instance_segmentation"), pallete: List[int] = const.COLOR_PALETTE) -> None:
         """
         Convert tasks to index color instance segmentation (PNG files).
-        Supports only bbox, polygon and segmentation annotation types. Hollowed points are not supported.
+        Supports only bbox, polygon and segmentation annotation types.
         Supports up to 57 instances in default colors palette. Check const.COLOR_PALETTE for more details.
 
         tasks is a list of tasks. (Required)
@@ -723,12 +1109,13 @@ class Client:
         """
         tasks = converters.to_pixel_coordinates(tasks)
         for task in tasks:
-            self.__export_index_color_image(task=task, output_dir=output_dir, pallete=pallete, is_instance_segmentation=True)
-    
+            self.__export_index_color_image(
+                task=task, output_dir=output_dir, pallete=pallete, is_instance_segmentation=True)
+
     def export_semantic_segmentation(self, tasks: list, output_dir: str = os.path.join("output", "semantic_segmentation"), pallete: List[int] = const.COLOR_PALETTE) -> None:
         """
         Convert tasks to index color semantic segmentation (PNG files).
-        Supports only bbox, polygon and segmentation annotation types. Hollowed points are not supported.
+        Supports only bbox, polygon and segmentation annotation types.
         Check const.COLOR_PALETTE for color pallete.
 
         tasks is a list of tasks. (Required)
@@ -744,7 +1131,8 @@ class Client:
 
         tasks = converters.to_pixel_coordinates(tasks)
         for task in tasks:
-            self.__export_index_color_image(task=task, output_dir=output_dir, pallete=pallete, is_instance_segmentation=False, classes=classes)
+            self.__export_index_color_image(
+                task=task, output_dir=output_dir, pallete=pallete, is_instance_segmentation=False, classes=classes)
 
     def __export_index_color_image(self, task: list, output_dir: str, pallete: List[int], is_instance_segmentation: bool = True, classes: list = []) -> None:
         image = Image.new("RGB", (task["width"], task["height"]), 0)
@@ -753,28 +1141,39 @@ class Client:
 
         index = 1
         for annotation in task["annotations"]:
-            color = index if is_instance_segmentation else classes.index(annotation["value"]) + 1
+            color = index if is_instance_segmentation else classes.index(
+                annotation["value"]) + 1
             if annotation["type"] == AnnotationType.segmentation.value:
                 for region in annotation["points"]:
                     count = 0
                     for points in region:
-                        cv_draw_points = self.__get_cv_draw_points(points)
                         if count == 0:
-                            cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
+                            cv_draw_points = self.__get_cv_draw_points(points)
+                            cv2.fillPoly(
+                                image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
                         else:
-                            cv2.fillPoly(image, [cv_draw_points], 0, lineType=cv2.LINE_8, shift=0)
+                            # Reverse hollow points for opencv because this points are counter clockwise
+                            cv_draw_points = self.__get_cv_draw_points(
+                                utils.reverse_points(points))
+                            cv2.fillPoly(
+                                image, [cv_draw_points], 0, lineType=cv2.LINE_8, shift=0)
                         count += 1
             elif annotation["type"] == AnnotationType.polygon.value:
-                cv_draw_points = self.__get_cv_draw_points(annotation["points"])
-                cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
+                cv_draw_points = self.__get_cv_draw_points(
+                    annotation["points"])
+                cv2.fillPoly(image, [cv_draw_points], color,
+                             lineType=cv2.LINE_8, shift=0)
             elif annotation["type"] == AnnotationType.bbox.value:
-                cv_draw_points = self.__get_cv_draw_points(annotation["points"])
-                cv2.fillPoly(image, [cv_draw_points], color, lineType=cv2.LINE_8, shift=0)
+                cv_draw_points = self.__get_cv_draw_points(
+                    annotation["points"])
+                cv2.fillPoly(image, [cv_draw_points], color,
+                             lineType=cv2.LINE_8, shift=0)
             else:
                 continue
             index += 1
 
-        image_path = os.path.join(output_dir, utils.get_basename(task["name"]) + ".png")
+        image_path = os.path.join(
+            output_dir, utils.get_basename(task["name"]) + ".png")
         os.makedirs(os.path.dirname(image_path), exist_ok=True)
         image = Image.fromarray(image)
         image = image.convert('P')
@@ -823,7 +1222,6 @@ class Client:
             cv_points.append((new_points[i * 2], new_points[i * 2 + 1]))
         return np.array(cv_points)
 
-
     # Annotation
 
     def find_annotation(self, annotation_id: str) -> dict:
@@ -1117,3 +1515,18 @@ class Client:
         """
         endpoint = "projects/" + project_id
         self.api.delete_request(endpoint)
+
+    @staticmethod
+    def __fill_assign_users(payload: dict, **kwargs):
+        if "assignee" in kwargs:
+            payload["assignee"] = kwargs.get("assignee")
+        if "reviewer" in kwargs:
+            payload["reviewer"] = kwargs.get("reviewer")
+        if "approver" in kwargs:
+            payload["approver"] = kwargs.get("approver")
+        if "external_assignee" in kwargs:
+            payload["externalAssignee"] = kwargs.get("external_assignee")
+        if "external_reviewer" in kwargs:
+            payload["externalReviewer"] = kwargs.get("external_reviewer")
+        if "external_approver" in kwargs:
+            payload["externalApprover"] = kwargs.get("external_approver")
