@@ -4,7 +4,9 @@ from typing import List
 import copy
 import geojson
 import numpy as np
+import math
 from fastlabel.const import AnnotationType
+import os
 
 # COCO
 
@@ -144,10 +146,12 @@ def __calc_area(annotation_type: str, points: list) -> float:
 # YOLO
 
 
-def to_yolo(tasks: list) -> tuple:
-    coco = to_coco(tasks)
-    yolo = __coco2yolo(coco)
-    return yolo
+def to_yolo(tasks: list, classes: list) -> tuple:
+    if len(classes) == 0:
+        coco = to_coco(tasks)
+        return __coco2yolo(coco)
+    else:
+        return __to_yolo(tasks, classes)
 
 
 def __coco2yolo(coco: dict) -> tuple:
@@ -198,6 +202,69 @@ def __coco2yolo(coco: dict) -> tuple:
     return annos, categories
 
 
+def __to_yolo(tasks: list, classes: list) -> tuple:
+    annos = []
+    for task in tasks:
+        if task["height"] == 0 or task["width"] == 0:
+            continue
+        objs = []
+        data = [{"annotation": annotation, "task": task, "classes": classes}
+                for annotation in task["annotations"]]
+        with ThreadPoolExecutor(max_workers=8) as executor:
+            results = executor.map(__get_yolo_annotation, data)
+            for result in results:
+                if not result:
+                    continue
+                objs.append(" ".join(result))
+            anno = {
+                "filename": task["name"],
+                "object": objs
+            }
+            annos.append(anno)
+
+    categories = map(lambda val: {"name": val}, classes)
+
+    return annos, categories
+
+
+def __get_yolo_annotation(data: dict) -> dict:
+    annotation = data["annotation"]
+    points = annotation["points"]
+    annotation_type = annotation["type"]
+    value = annotation["value"]
+    classes = list(data["classes"])
+    task = data["task"]
+    if annotation_type != AnnotationType.bbox.value and annotation_type != AnnotationType.polygon.value:
+        return None
+    if not points or len(points) == 0:
+        return None
+    if annotation_type == AnnotationType.bbox.value and (int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])):
+        return None
+    if not annotation["value"] in classes:
+        return None
+
+    dw = 1. / task["width"]
+    dh = 1. / task["height"]
+
+    bbox = __to_bbox(points)
+    xmin = bbox[0]
+    ymin = bbox[1]
+    xmax = bbox[0] + bbox[2]
+    ymax = bbox[1] + bbox[3]
+
+    x = (xmin + xmax) / 2
+    y = (ymin + ymax) / 2
+    w = xmax - xmin
+    h = ymax - ymin
+
+    x = str(_truncate(x * dw, 7))
+    y = str(_truncate(y * dh, 7))
+    w = str(_truncate(w * dw, 7))
+    h = str(_truncate(h * dh, 7))
+    category_index = str(classes.index(value))
+    return [category_index, x, y, w, h]
+
+
 def _truncate(n, decimals=0) -> float:
     multiplier = 10 ** decimals
     return int(n * multiplier) / multiplier
@@ -207,10 +274,75 @@ def _truncate(n, decimals=0) -> float:
 
 
 def to_pascalvoc(tasks: list) -> list:
-    coco = to_coco(tasks)
-    pascalvoc = __coco2pascalvoc(coco)
+    pascalvoc = []
+    for task in tasks:
+        if task["height"] == 0 or task["width"] == 0:
+            continue
+
+        pascal_objs = []
+        data = [{"annotation": annotation}
+                for annotation in task["annotations"]]
+        with ThreadPoolExecutor(max_workers=8) as executor:
+            results = executor.map(__get_pascalvoc_obj, data)
+
+        for result in results:
+            if not result:
+                continue
+            pascal_objs.append(result)
+
+        voc = {
+            "annotation": {
+                "filename": task["name"],
+                "size": {
+                    "width": task["width"],
+                    "height": task["height"],
+                    "depth": 3,
+                },
+                "segmented": 0,
+                "object": pascal_objs
+            }
+        }
+        pascalvoc.append(voc)
     return pascalvoc
 
+def __get_pascalvoc_obj(data: dict) -> dict:
+    annotation = data["annotation"]
+    points = annotation["points"]
+    annotation_type = annotation["type"]
+    if annotation_type != AnnotationType.bbox.value and annotation_type != AnnotationType.polygon.value:
+        return None
+    if not points or len(points) == 0:
+        return None
+    if annotation_type == AnnotationType.bbox.value and (int(points[0]) == int(points[2]) or int(points[1]) == int(points[3])):
+        return None
+    bbox = __to_bbox(points)
+    x = bbox[0]
+    y = bbox[1]
+    w = bbox[2]
+    h = bbox[3]
+
+    return {
+        "name": annotation["value"],
+        "pose": "Unspecified",
+                "truncated": __get_pascalvoc_tag_value(annotation, "truncated"),
+                "occluded": __get_pascalvoc_tag_value(annotation, "occluded"),
+                "difficult": __get_pascalvoc_tag_value(annotation, "difficult"),
+                "bndbox": {
+                    "xmin": math.floor(x),
+                    "ymin": math.floor(y),
+                    "xmax": math.floor(x + w),
+                    "ymax": math.floor(y + h),
+        },
+    }
+
+def __get_pascalvoc_tag_value(annotation: dict, target_tag_name: str) -> int:
+    attributes = annotation["attributes"]
+    if not attributes:
+        return 0
+    related_attr = next(
+        (attribute for attribute in attributes if attribute["type"] == "switch" and attribute["key"] == target_tag_name), None)
+    return int(related_attr["value"]) if related_attr else 0
+
 
 # labelme
 
@@ -389,57 +521,166 @@ def __get_pixel_coordinates(points: List[int or float]) -> List[int]:
                 new_points.append(int(prev_y + int(ydiff / mindiff * (i + 1))))
     return new_points
 
-def __coco2pascalvoc(coco: dict) -> list:
-    pascalvoc = []
 
-    for image in coco["images"]:
+def execute_coco_to_fastlabel(coco: dict) -> dict:
+    coco_images = {}
+    for c in coco["images"]:
+        coco_images[c["id"]] = c["file_name"]
 
-        # Get objects
-        objs = []
-        for annotation in coco["annotations"]:
-            if image["id"] != annotation["image_id"]:
-                continue
-            category = _get_category_by_id(
-                coco["categories"], annotation["category_id"])
+    coco_categories = {}
+    for c in coco["categories"]:
+        coco_categories[c["id"]] = c["name"]
 
-            x = annotation["bbox"][0]
-            y = annotation["bbox"][1]
-            w = annotation["bbox"][2]
-            h = annotation["bbox"][3]
+    coco_annotations = coco["annotations"]
 
-            obj = {
-                "name": category["name"],
-                "pose": "Unspecified",
-                "truncated": 0,
-                "difficult": 0,
-                "bndbox": {
-                        "xmin": x,
-                        "ymin": y,
-                        "xmax": x + w,
-                        "ymax": y + h,
-                },
+    results = {}
+    for coco_image_key in coco_images:
+        target_coco_annotations = filter(
+            lambda annotation: annotation["image_id"] == coco_image_key,
+            coco_annotations,
+        )
+        if not target_coco_annotations:
+            return
+
+        annotations = []
+        for target_coco_annotation in target_coco_annotations:
+            category_name = coco_categories[target_coco_annotation["category_id"]]
+            if not category_name:
+                return
+
+            segmentation = target_coco_annotation["segmentation"][0]
+            annotation_type = ""
+            if len(segmentation) == 4:
+                annotation_type = AnnotationType.bbox.value
+            if len(segmentation) > 4:
+                annotation_type = AnnotationType.polygon.value
+            annotations.append(
+                {
+                    "value": category_name,
+                    "points": segmentation,
+                    "type": annotation_type,
+                }
+            )
+        results[coco_images[coco_image_key]] = annotations
+    return results
+
+def execute_labelme_to_fastlabel(labelme: dict, file_path: str = None) -> tuple:
+    file_name = ""
+    if file_path:
+        file_name = file_path.replace(
+            ".json", os.path.splitext(labelme["imagePath"])[1]
+        )
+    else:
+        file_name = labelme["imagePath"]
+    labelme_annotations = labelme["shapes"]
+
+    annotations = []
+    for labelme_annotation in labelme_annotations:
+        label = labelme_annotation["label"]
+        if not label:
+            return
+
+        points = np.ravel(labelme_annotation["points"])
+        annotation_type = __get_annotation_type_by_labelme(
+            labelme_annotation["shape_type"]
+        )
+        annotations.append(
+            {"value": label, "points": points.tolist(), "type": annotation_type}
+        )
+
+    return (file_name, annotations)
+
+def execute_pascalvoc_to_fastlabel(pascalvoc: dict, file_path: str = None) -> tuple:
+    target_pascalvoc = pascalvoc["annotation"]
+    file_name = ""  # file_path if file_path else target_pascalvoc["filename"]
+    if file_path:
+        file_name = file_path.replace(
+            ".xml", os.path.splitext(target_pascalvoc["filename"])[1]
+        )
+    else:
+        file_name = target_pascalvoc["filename"]
+    pascalvoc_annotations = target_pascalvoc["object"]
+    if not isinstance(pascalvoc_annotations, list):
+        pascalvoc_annotations = [pascalvoc_annotations]
+
+    annotations = []
+    for pascalvoc_annotation in pascalvoc_annotations:
+        category_name = pascalvoc_annotation["name"]
+        if not category_name:
+            return
+
+        points = [
+            int(pascalvoc_annotation["bndbox"][item])
+            for item in pascalvoc_annotation["bndbox"]
+        ]
+        annotations.append(
+            {
+                "value": category_name,
+                "points": points,
+                "type": AnnotationType.bbox.value,
             }
-            objs.append(obj)
+        )
 
-        # get annotation
-        voc = {
-            "annotation": {
-                "filename": image["file_name"],
-                "size": {
-                    "width": image["width"],
-                    "height": image["height"],
-                    "depth": 3,
-                },
-                "segmented": 0,
-                "object": objs
-            }
-        }
-        pascalvoc.append(voc)
-
-    return pascalvoc
+    return (file_name, annotations)
 
 
-def _get_category_by_id(categories: list, id_: str) -> str:
-    category = [
-        category for category in categories if category["id"] == id_][0]
-    return category
+def execute_yolo_to_fastlabel(
+    classes: dict,
+    image_sizes: dict,
+    yolo_annotations: dict,
+    dataset_folder_path: str = None,
+) -> dict:
+    results = {}
+    for yolo_anno_key in yolo_annotations:
+        annotations = []
+        for each_image_annotation in yolo_annotations[yolo_anno_key]:
+            (
+                yolo_class_id,
+                yolo_center_x_ratio,
+                yolo_center_y_ratio,
+                yolo_anno_width_ratio,
+                yolo_anno_height_ratio,
+            ) = each_image_annotation
+            image_width, image_height = image_sizes[yolo_anno_key]["size"]
+
+            classs_name = classes[str(yolo_class_id)]
+
+            yolo_center_x_point = float(image_width) * float(yolo_center_x_ratio)
+            yolo_center_y_point = float(image_height) * float(yolo_center_y_ratio)
+            yolo_anno_width_size = float(image_width) * float(yolo_anno_width_ratio)
+            yolo_anno_height_size = float(image_height) * float(yolo_anno_height_ratio)
+
+            points = []
+            points.append(yolo_center_x_point - (yolo_anno_width_size / 2))  # x1
+            points.append(yolo_center_y_point - (yolo_anno_height_size / 2))  # y1
+            points.append(yolo_center_x_point + (yolo_anno_width_size / 2))  # x2
+            points.append(yolo_center_y_point + (yolo_anno_height_size / 2))  # y2
+            annotations.append(
+                {
+                    "value": classs_name,
+                    "points": points,
+                    "type": AnnotationType.bbox.value,
+                }
+            )
+
+        file_path = (
+            image_sizes[yolo_anno_key]["image_file_path"].replace(
+                os.path.join(*[dataset_folder_path, ""]), ""
+            )
+            if dataset_folder_path
+            else image_sizes[yolo_anno_key]["image_file_path"]
+        )
+        results[file_path] = annotations
+
+    return results
+
+def __get_annotation_type_by_labelme(shape_type: str) -> str:
+    if shape_type == "rectangle":
+        return "bbox"
+    if shape_type == "polygon":
+        return "polygon"
+    if shape_type == "point":
+        return "keypoint"
+    if shape_type == "line":
+        return "line"
+    return None
